package com.zhiyun.project.system.controller;

import com.zhiyun.common.annotation.Log;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.enums.BusinessType;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.model.ApiResult;
import com.zhiyun.common.model.TableResult;
import com.zhiyun.framework.RoleDataScope;
import com.zhiyun.framework.web.SysPermissionService;
import com.zhiyun.framework.web.TokenService;
import com.zhiyun.project.common.BaseController;
import com.zhiyun.project.system.domain.body.SysDeptBody;
import com.zhiyun.project.system.domain.body.SysRoleBody;
import com.zhiyun.project.system.domain.body.SysUserBody;
import com.zhiyun.project.system.domain.dto.LoginUser;
import com.zhiyun.project.system.domain.entity.SysRole;
import com.zhiyun.project.system.domain.entity.SysUserRole;
import com.zhiyun.project.system.service.ISysDeptService;
import com.zhiyun.project.system.service.ISysRoleService;
import com.zhiyun.project.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

/**
 * 角色信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/role")
public class SysRoleController extends BaseController {
    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysDeptService deptService;

    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @GetMapping("/list")
    public TableResult pageRole(SysRoleBody role) {
        return TableResult.getTableResult(roleService.pageRole(role));
    }

    /**
     * 根据角色编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping(value = "/{roleId}")
    public ApiResult getInfo(@PathVariable Long roleId) {
        roleService.checkRoleDataScope(roleId);
        return ApiResult.success(roleService.getById(roleId));
    }

    /**
     * 新增角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:add')")
    @Log(title = "角色管理", businessType = BusinessType.INSERT)
    @PostMapping
    public ApiResult add(@Validated @RequestBody SysRole role) {
        if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleNameUnique(role))) {
            return ApiResult.error("新增角色'" + role.getRoleName() + "'失败，角色名称已存在");
        } else if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleKeyUnique(role))) {
            return ApiResult.error("新增角色'" + role.getRoleName() + "'失败，角色权限已存在");
        }
        role.setCreateBy(getUsername());
        return toApiResult(roleService.insertRole(role));

    }

    /**
     * 修改保存角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public ApiResult edit(@Validated @RequestBody SysRole role) {
        if (role.getRoleId() != null && role.getRoleId() == 1) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
        roleService.checkRoleDataScope(role.getRoleId());
        if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleNameUnique(role))) {
            return ApiResult.error("修改角色'" + role.getRoleName() + "'失败，角色名称已存在");
        } else if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleKeyUnique(role))) {
            return ApiResult.error("修改角色'" + role.getRoleName() + "'失败，角色权限已存在");
        }
        role.setUpdateBy(getUsername());

        if (roleService.updateRole(role)) {
            // 更新缓存用户权限
            LoginUser loginUser = getLoginUser();
            if (loginUser != null && !RoleDataScope.isAdmin(loginUser.getUserId())) {
                loginUser.setMenuPermissions(permissionService.getMenuPermissionByUserId(loginUser.getUserId()));
                loginUser.setUser(userService.getById(loginUser.getUserId()));
                tokenService.setLoginUser(loginUser);
            }
            return ApiResult.success();
        }
        return ApiResult.error("修改角色'" + role.getRoleName() + "'失败，请联系管理员");
    }

    /**
     * 修改保存数据权限
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @PutMapping("/dataScope")
    public ApiResult dataScope(@RequestBody SysRole role) {
        if (role.getRoleId() != null && role.getRoleId() == 1) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
        roleService.checkRoleDataScope(role.getRoleId());
        return toApiResult(roleService.authDataScope(role));
    }

    /**
     * 状态修改
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public ApiResult changeStatus(@RequestBody SysRole role) {
        if (role.getRoleId() != null && role.getRoleId() == 1) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
        roleService.checkRoleDataScope(role.getRoleId());
        role.setUpdateBy(getUsername());
        return toApiResult(roleService.updateRoleStatus(role));
    }

    /**
     * 删除角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:remove')")
    @Log(title = "角色管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{roleIds}")
    public ApiResult remove(@PathVariable Long[] roleIds) {
        return toApiResult(roleService.deleteRoleByIds(roleIds));
    }

    /**
     * 获取角色选择框列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping("/optionselect")
    public ApiResult optionselect() {
        return ApiResult.success(roleService.list());
    }

    /**
     * 查询已分配用户角色列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @GetMapping("/authUser/allocatedList")
    public TableResult allocatedList(SysUserBody user) {
        return TableResult.getTableResult(userService.selectAllocatedList(user));
    }

    /**
     * 查询未分配用户角色列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @GetMapping("/authUser/unallocatedList")
    public TableResult unallocatedList(SysUserBody user) {
        return TableResult.getTableResult(userService.selectUnallocatedList(user));
    }

    /**
     * 取消授权用户
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @PutMapping("/authUser/cancel")
    public ApiResult cancelAuthUser(@RequestBody SysUserRole userRole) {
        return toApiResult(roleService.deleteAuthUser(userRole));
    }

    /**
     * 批量取消授权用户
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @PutMapping("/authUser/cancelAll")
    public ApiResult cancelAuthUserAll(Long roleId, Long[] userIds) {
        return toApiResult(roleService.deleteAuthUsers(roleId, userIds));
    }

    /**
     * 批量选择用户授权
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @PutMapping("/authUser/selectAll")
    public ApiResult selectAuthUserAll(Long roleId, Long[] userIds) {
        roleService.checkRoleDataScope(roleId);
        return toApiResult(roleService.insertAuthUsers(roleId, userIds));
    }

    /**
     * 获取对应角色部门树列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping(value = "/deptTree/{roleId}")
    public ApiResult deptTree(@PathVariable("roleId") Long roleId) {
        Map<String, Object> result = new HashMap<>();
        result.put("checkedKeys", deptService.selectDeptListByRoleId(roleId));
        result.put("depts", deptService.selectDeptTreeList(new SysDeptBody()));
        return ApiResult.success(result);
    }
}
