package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiyun.common.constant.CacheConstants;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.core.redis.RedisCache;
import com.zhiyun.common.core.text.Convert;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.project.system.domain.body.SysConfigBody;
import com.zhiyun.project.system.domain.entity.SysConfig;
import com.zhiyun.project.system.mapper.SysConfigMapper;
import com.zhiyun.project.system.service.ISysConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Collection;
import java.util.List;

/**
 * 参数配置 服务层实现
 *
 * @author ruoyi
 */
@Service
public class SysConfigServiceImpl extends ServiceImpl<SysConfigMapper, SysConfig> implements ISysConfigService {

    @Autowired
    private RedisCache redisCache;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init() {
        loadingConfigCache();
    }

    @Override
    public String selectConfigByKey(String configKey) {
        //优先查缓存
        String configValue = Convert.toStr(redisCache.getCacheObject(getCacheKey(configKey)));
        if (StringUtils.isNotEmpty(configValue)) {
            return configValue;
        }
        //查询数据库
        SysConfig retConfig = baseMapper.selectOne(new QueryWrapper<SysConfig>().eq("config_key", configKey));
        if (StringUtils.isNotNull(retConfig)) {
            redisCache.setCacheObject(getCacheKey(configKey), retConfig.getConfigValue());
            return retConfig.getConfigValue();
        }
        return StringUtils.EMPTY;
    }

    @Override
    public Page<SysConfig> selectConfigPage(SysConfigBody body) {
        Page<SysConfig> page = new Page<>(body.getPageNum(), body.getPageSize());
        return baseMapper.selectConfigList(page, body);
    }

    @Override
    public boolean insertConfig(SysConfig config) {
        if (save(config)) {
            redisCache.setCacheObject(getCacheKey(config.getConfigKey()), config.getConfigValue());
            return true;
        }
        return false;
    }

    @Override
    public boolean updateConfig(SysConfig config) {
        SysConfig temp = baseMapper.selectById(config.getConfigId());
        //如果键值不同,则删除缓存
        if (!StringUtils.equals(temp.getConfigKey(), config.getConfigKey())) {
            redisCache.deleteObject(getCacheKey(temp.getConfigKey()));
        }
        if (updateById(config)) {
            redisCache.setCacheObject(getCacheKey(config.getConfigKey()), config.getConfigValue());
            return true;
        }
        return false;
    }

    @Override
    public void deleteConfigByIds(Long[] configIds) {
        for (Long configId : configIds) {
            SysConfig config = getById(configId);
            if (StringUtils.equals(UserConstants.YES, config.getConfigType())) {
                throw new ServiceException(String.format("内置参数【%1$s】不能删除 ", config.getConfigKey()));
            }
            removeById(configId);
            redisCache.deleteObject(getCacheKey(config.getConfigKey()));
        }
    }

    @Override
    public void resetConfigCache() {
        clearConfigCache();
        loadingConfigCache();
    }

    @Override
    public String checkConfigKeyUnique(Long configId, String configKey) {
        List<SysConfig> list = list(new QueryWrapper<SysConfig>().eq("config_key", configKey));
        if (list.isEmpty() || list.get(0).getConfigId().equals(configId)) return UserConstants.UNIQUE;
        return UserConstants.NOT_UNIQUE;
    }

    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    private String getCacheKey(String configKey) {
        return CacheConstants.SYS_CONFIG_KEY + configKey;
    }

    /**
     * 加载全部的配置信息
     */
    private void loadingConfigCache() {
        List<SysConfig> configsList = list();
        for (SysConfig config : configsList) {
            redisCache.setCacheObject(getCacheKey(config.getConfigKey()), config.getConfigValue());
        }
    }

    /**
     * 清空参数缓存数据
     */
    private void clearConfigCache() {
        Collection<String> keys = redisCache.keys(CacheConstants.SYS_CONFIG_KEY + "*");
        redisCache.deleteObject(keys);
    }
}
