package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.core.text.Convert;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.utils.SecurityUtils;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.framework.RoleDataScope;
import com.zhiyun.project.system.domain.body.SysDeptBody;
import com.zhiyun.project.system.domain.entity.SysDept;
import com.zhiyun.project.system.domain.entity.SysUser;
import com.zhiyun.project.system.domain.vo.SysDeptVo;
import com.zhiyun.project.system.domain.vo.TreeSelect;
import com.zhiyun.project.system.mapper.SysDeptMapper;
import com.zhiyun.project.system.service.ISysDeptService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 *
 * @author ruoyi
 */
@Service
public class SysDeptServiceImpl extends ServiceImpl<SysDeptMapper, SysDept> implements ISysDeptService {

    @Override
    public List<SysDeptVo> selectDeptList(SysDeptBody body) {
        return baseMapper.selectDeptList(RoleDataScope.getDataScopePerm(), body);
    }

    @Override
    public List<TreeSelect> selectDeptTreeList(SysDeptBody body) {
        List<SysDeptVo> deptList = baseMapper.selectDeptList(RoleDataScope.getDataScopePerm(), body);
        List<SysDeptVo> deptTrees = new ArrayList<>();
        Set<Long> deptIdSet = deptList.stream().map(SysDeptVo::getDeptId).collect(Collectors.toSet());
        for (SysDeptVo dept : deptList) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!deptIdSet.contains(dept.getParentId())) {
                recurveFillChildDept(deptList, dept);
                deptTrees.add(dept);
            }
        }
        if (deptTrees.isEmpty()) {
            deptTrees = deptList;
        }
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    @Override
    public List<Long> selectDeptListByRoleId(Long roleId) {
        return baseMapper.selectDeptListByRoleId(roleId);
    }

    @Override
    public SysDeptVo selectDeptById(Long deptId) {
        return baseMapper.selectDeptById(deptId);
    }

    @Override
    public int countNormalChildDeptById(Long deptId) {
        return baseMapper.countNormalChildDeptById(deptId);
    }

    @Override
    public boolean hasChildByDeptId(Long deptId) {
        int result = baseMapper.hasChildByDeptId(deptId);
        return result > 0;
    }

    @Override
    public boolean checkDeptExistUser(Long deptId) {
        return baseMapper.checkDeptExistUser(deptId) > 0;
    }

    @Override
    public String checkDeptNameUnique(SysDept dept) {
        long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        Long dbDeptId = baseMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (dbDeptId != null && dbDeptId != deptId) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public void checkDeptDataScope(Long deptId) {
        SysUser crtUser = SecurityUtils.getLoginUser().getUser();
        if (crtUser == null) throw new ServiceException("没有权限访问部门数据！");
        if (!RoleDataScope.isAdmin(crtUser.getUserId())) {
            SysDeptBody dept = new SysDeptBody();
            dept.setDeptId(deptId);
            List<SysDeptVo> deptList = selectDeptList(dept);
            if (StringUtils.isEmpty(deptList)) {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    @Override
    public int insertDept(SysDept dept) {
        SysDeptVo parent = baseMapper.selectDeptById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(parent.getStatus())) {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(parent.getAncestors() + "," + dept.getParentId());
        return baseMapper.insert(dept);
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int updateDept(SysDept dept) {
        //父级部门
        SysDept newParentDept = getById(dept.getParentId());
        //数据库中旧数据
        SysDept oldDept = getById(dept.getDeptId());
        if (newParentDept != null && oldDept != null) {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            dept.setAncestors(newAncestors);
            //修改子部门的 祖级列表
            updateDeptChildren(dept.getDeptId(), newAncestors, oldDept.getAncestors());
        }
        //修改部门信息
        int result = baseMapper.updateById(dept);

        // 如果该部门是启用状态，则启用该部门的所有上级部门
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors()) && !StringUtils.equals("0", dept.getAncestors())) {
            updateParentDeptStatusNormal(dept);
        }
        return result;
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(SysDept dept) {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        baseMapper.updateDeptStatusNormal(deptIds);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId       被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    private void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors) {
        List<SysDept> children = baseMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children) {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (!children.isEmpty()) {
            baseMapper.updateDeptChildren(children);
        }
    }

    @Override
    public int deleteDeptById(Long deptId) {
        return baseMapper.deleteDeptById(deptId);
    }

    /**
     * 递归填充子菜单
     *
     * @param allList 全部的部门
     * @param dept    部门
     */
    private void recurveFillChildDept(List<SysDeptVo> allList, SysDeptVo dept) {
        // 得到子节点列表
        List<SysDeptVo> childList = getChildList(allList, dept.getDeptId());
        if (childList.isEmpty()) return;
        dept.setChildren(childList);
        for (SysDeptVo child : childList) {
            recurveFillChildDept(allList, child);
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDeptVo> getChildList(List<SysDeptVo> list, Long parentId) {
        return list.stream().filter(item -> item.getParentId().longValue() == parentId.longValue()).collect(Collectors.toList());
    }
}
