package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.utils.DictUtils;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.project.system.domain.body.SysDictBody;
import com.zhiyun.project.system.domain.entity.SysDictData;
import com.zhiyun.project.system.domain.entity.SysDictType;
import com.zhiyun.project.system.mapper.SysDictDataMapper;
import com.zhiyun.project.system.mapper.SysDictTypeMapper;
import com.zhiyun.project.system.service.ISysDictTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字典 业务层处理
 *
 * @author ruoyi
 */
@Service
public class SysDictTypeServiceImpl extends ServiceImpl<SysDictTypeMapper, SysDictType> implements ISysDictTypeService {

    @Autowired
    private SysDictDataMapper dictDataMapper;

    /**
     * 项目启动时，初始化字典到缓存
     */
    @PostConstruct
    public void init() {
        loadingDictCache();
    }

    @Override
    public Page<SysDictType> selectDictTypePage(SysDictBody body) {
        return baseMapper.selectDictTypeList(new Page<>(body.getPageNum(), body.getPageSize()), body);
    }

    @Override
    public List<SysDictData> selectDictDataByType(String dictType) {
        List<SysDictData> dictDatas = DictUtils.getDictCache(dictType);
        if (StringUtils.isNotEmpty(dictDatas)) {
            return dictDatas;
        }
        dictDatas = dictDataMapper.selectDictDataByType(dictType);
        if (StringUtils.isNotEmpty(dictDatas)) {
            DictUtils.setDictCache(dictType, dictDatas);
            return dictDatas;
        }
        return null;
    }

    @Override
    public void deleteDictTypeByIds(Long[] dictIds) {
        for (Long dictId : dictIds) {
            SysDictType dictType = baseMapper.selectById(dictId);
            if (dictDataMapper.countDictDataByType(dictType.getDictType()) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", dictType.getDictName()));
            }
            baseMapper.deleteById(dictId);
            DictUtils.removeDictCache(dictType.getDictType());
        }
    }

    @Override
    public void resetDictCache() {
        //清除缓存
        DictUtils.clearDictCache();
        loadingDictCache();
    }

    /**
     * 加载字典缓存数据
     */
    private void loadingDictCache() {
        //筛选全部有效字典数据
        List<SysDictData> dictDataList = dictDataMapper.selectList(new QueryWrapper<SysDictData>().eq("status", "0"));
        //保存字典数据
        dictDataList.stream().collect(Collectors.groupingBy(SysDictData::getDictType))
                .forEach((key, value) -> DictUtils.setDictCache(key, value.stream().sorted(Comparator.comparing(SysDictData::getDictSort)).collect(Collectors.toList())));
    }

    @Override
    public boolean insertDictType(SysDictType dict) {
        if (baseMapper.insert(dict) > 0) {
            DictUtils.setDictCache(dict.getDictType(), null);
            return true;
        }
        return false;
    }

    @Override
    @Transactional
    public boolean updateDictType(SysDictType dict) {
        SysDictType oldDict = baseMapper.selectById(dict.getDictId());
        dictDataMapper.updateDictDataType(oldDict.getDictType(), dict.getDictType());
        if (baseMapper.updateById(dict) > 0) {
            List<SysDictData> dictDataList = dictDataMapper.selectDictDataByType(dict.getDictType());
            DictUtils.setDictCache(dict.getDictType(), dictDataList);
            return true;
        }
        return false;
    }

    @Override
    public String checkDictTypeUnique(SysDictType dict) {
        long dictId = StringUtils.isNull(dict.getDictId()) ? -1L : dict.getDictId();
        Long dbDictId = baseMapper.checkDictTypeUnique(dict.getDictType());
        if (dbDictId != null && dbDictId != dictId) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
}
