package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zhiyun.common.constant.CacheConstants;
import com.zhiyun.common.constant.Constants;
import com.zhiyun.common.core.redis.RedisCache;
import com.zhiyun.common.exception.UserException;
import com.zhiyun.common.utils.ServletUtils;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.common.utils.ip.IpUtils;
import com.zhiyun.framework.manager.AsyncManager;
import com.zhiyun.framework.manager.factory.AsyncFactory;
import com.zhiyun.framework.web.TokenService;
import com.zhiyun.project.system.domain.dto.LoginUser;
import com.zhiyun.project.system.domain.entity.SysUser;
import com.zhiyun.project.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 登录校验方法
 *
 * @author ruoyi
 */
@Component
public class SysLoginService {
    @Autowired
    private TokenService tokenService;
    @Resource
    private AuthenticationManager authenticationManager;
    @Autowired
    private ISysUserService userService;
    @Autowired
    private RedisCache redisCache;

    @Value(value = "${user.password.maxRetryCount}")
    private int maxRetryCount;

    @Value(value = "${user.password.lockTime}")
    private int lockTime;

    /**
     * 登录验证
     *
     * @param username 用户名
     * @param password 密码
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid) {
        // 验证码校验
        validateCaptcha(username, code, uuid);
        //校验登录次数
        validateRetryCount(username);
        try {
            // 用户验证 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            Authentication authentication = authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(username, password));
            LoginUser loginUser = (LoginUser) authentication.getPrincipal();
            //处理登录成功
            handleLoginSuccess(loginUser);
            // 生成token
            return tokenService.createToken(loginUser);
        } catch (Exception e) {
            //重试次数++
            retryCountIncrement(username);
            if (e instanceof BadCredentialsException) {
                String message = "密码错误";
                AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, message));
                throw new UserException(message);
            } else {
                AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new UserException(e.getMessage());
            }
        }
    }

    /**
     * 登陆成功
     */
    private void handleLoginSuccess(LoginUser user) {
        //记录登录信息
        userService.update(new UpdateWrapper<SysUser>().set("login_ip", IpUtils.getIpAddress(ServletUtils.getRequest())).set("login_date", new Date()).eq("user_id", user.getUserId()));
        //记录日志
        AsyncManager.me().execute(AsyncFactory.recordLoginInfo(user.getUsername(), Constants.LOGIN_SUCCESS, "登录成功"));
        //重置登录次数
        redisCache.deleteObject(getCacheKey(user.getUsername()));
    }

    /**
     * 校验验证码
     *
     * @param username 用户名
     * @param code     验证码
     * @param uuid     唯一标识
     */
    private void validateCaptcha(String username, String code, String uuid) {
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
        String captcha = redisCache.getCacheObject(verifyKey);
        redisCache.deleteObject(verifyKey);
        if (captcha == null) {
            AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, "验证码已失效"));
            throw new UserException("验证码失效");
        }
        if (!code.equalsIgnoreCase(captcha)) {
            AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, "验证码错误"));
            throw new UserException("验证码错误");
        }
    }


    /**
     * 校验重试次数
     */
    private void validateRetryCount(String username) {
        Integer retryCount = redisCache.getCacheObject(getCacheKey(username));
        if (retryCount != null && retryCount >= maxRetryCount) {
            String message = StringUtils.format("密码输入错误{}次，帐户锁定{}分钟", maxRetryCount, lockTime);
            AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, message));
            throw new UserException(message);
        }
    }

    /**
     * 重试次数++
     */
    public void retryCountIncrement(String username) {
        Integer retryCount = redisCache.getCacheObject(getCacheKey(username));
        if (retryCount == null) retryCount = 0;
        AsyncManager.me().execute(AsyncFactory.recordLoginInfo(username, Constants.LOGIN_FAIL, StringUtils.format("密码输入错误{}次", retryCount)));
        redisCache.setCacheObject(getCacheKey(username), ++retryCount, lockTime, TimeUnit.MINUTES);
    }

    /**
     * 登录账户密码错误次数缓存键名
     *
     * @param username 用户名
     * @return 缓存键key
     */
    private String getCacheKey(String username) {
        return CacheConstants.PWD_ERR_CNT_KEY + username;
    }

}
