package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.utils.SecurityUtils;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.framework.RoleDataScope;
import com.zhiyun.project.system.domain.body.SysRoleBody;
import com.zhiyun.project.system.domain.entity.SysRole;
import com.zhiyun.project.system.domain.entity.SysRoleDept;
import com.zhiyun.project.system.domain.entity.SysRoleMenu;
import com.zhiyun.project.system.domain.entity.SysUserRole;
import com.zhiyun.project.system.mapper.SysRoleDeptMapper;
import com.zhiyun.project.system.mapper.SysRoleMapper;
import com.zhiyun.project.system.mapper.SysRoleMenuMapper;
import com.zhiyun.project.system.mapper.SysUserRoleMapper;
import com.zhiyun.project.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 角色 业务层处理
 *
 * @author ruoyi
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements ISysRoleService {

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleDeptMapper roleDeptMapper;

    @Override
    public Page<SysRole> pageRole(SysRoleBody body) {
        return baseMapper.pageRole(body.getPage(), body);
    }

    @Override
    public List<SysRole> selectRolesByUserId(Long userId) {
        return baseMapper.selectRoleByUserId(userId);
    }

    @Override
    public Set<String> selectRolePermissionByUserId(Long userId) {
        List<SysRole> perms = baseMapper.selectRoleByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (perm != null) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }


    @Override
    public String checkRoleNameUnique(SysRole role) {
        long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        Long dbRoleId = baseMapper.checkRoleNameUnique(role.getRoleName());
        if (dbRoleId != null && dbRoleId != roleId) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public String checkRoleKeyUnique(SysRole role) {
        long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        Long dbRoleId = baseMapper.checkRoleKeyUnique(role.getRoleKey());
        if (dbRoleId != null && dbRoleId != roleId) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public void checkRoleDataScope(Long roleId) {
        if (!RoleDataScope.isAdmin(SecurityUtils.getUserId())) {
            throw new ServiceException("没有权限访问角色数据！");
        }
    }

    @Override
    public int countUserRoleByRoleId(Long roleId) {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    @Override
    @Transactional
    public boolean insertRole(SysRole role) {
        // 新增角色信息
        baseMapper.insert(role);

        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<>();
        for (Long menuId : role.getMenuIds()) {
            list.add(new SysRoleMenu(role.getRoleId(), menuId));
        }
        return !list.isEmpty() && roleMenuMapper.batchRoleMenu(list) == list.size();
    }

    @Override
    @Transactional
    public boolean updateRole(SysRole role) {
        // 修改角色信息
        baseMapper.updateById(role);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());

        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<>();
        for (Long menuId : role.getMenuIds()) {
            list.add(new SysRoleMenu(role.getRoleId(), menuId));
        }
        return !list.isEmpty() && roleMenuMapper.batchRoleMenu(list) == list.size();
    }

    @Override
    public int updateRoleStatus(SysRole role) {
        return baseMapper.updateById(role);
    }

    @Override
    @Transactional
    public boolean authDataScope(SysRole role) {
        // 修改角色信息
        baseMapper.updateById(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        if (role.getDeptIds().length != 0) {
            // 新增角色与部门（数据权限）管理
            List<SysRoleDept> list = new ArrayList<>();
            for (Long deptId : role.getDeptIds()) {
                list.add(new SysRoleDept(role.getRoleId(), deptId));
            }
            roleDeptMapper.batchRoleDept(list);
        }
        return true;
    }

    @Override
    @Transactional
    public int deleteRoleByIds(Long[] roleIds) {
        for (Long roleId : roleIds) {
            if (roleId != null && roleId == 1) {
                throw new ServiceException("不允许操作超级管理员角色");
            }
            checkRoleDataScope(roleId);
            SysRole role = getById(roleId);
            if (countUserRoleByRoleId(roleId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDept(roleIds);
        return baseMapper.deleteRoleByIds(roleIds);
    }

    @Override
    public int deleteAuthUser(SysUserRole userRole) {
        return userRoleMapper.deleteUserRoleInfo(userRole);
    }

    @Override
    public boolean deleteAuthUsers(Long roleId, Long[] userIds) {
        return userRoleMapper.deleteUserRoleInfos(roleId, userIds) > 0;
    }

    @Override
    public int insertAuthUsers(Long roleId, Long[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<>();
        for (Long userId : userIds) {
            list.add(new SysUserRole(userId, roleId));
        }
        return userRoleMapper.batchUserRole(list);
    }
}
