package com.zhiyun.project.system.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiyun.common.constant.UserConstants;
import com.zhiyun.common.exception.ServiceException;
import com.zhiyun.common.utils.SecurityUtils;
import com.zhiyun.common.utils.StringUtils;
import com.zhiyun.framework.RoleDataScope;
import com.zhiyun.project.item.domain.dto.Perm;
import com.zhiyun.project.system.domain.body.SysUserBody;
import com.zhiyun.project.system.domain.entity.SysRole;
import com.zhiyun.project.system.domain.entity.SysUser;
import com.zhiyun.project.system.domain.entity.SysUserRole;
import com.zhiyun.project.system.domain.vo.SysUserVo;
import com.zhiyun.project.system.domain.vo.UserActiveVo;
import com.zhiyun.project.system.mapper.SysRoleMapper;
import com.zhiyun.project.system.mapper.SysUserMapper;
import com.zhiyun.project.system.mapper.SysUserRoleMapper;
import com.zhiyun.project.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户 业务层处理
 *
 * @author ruoyi
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Override
    public Page<SysUserVo> selectUserPage(SysUserBody body) {
        Perm perm = RoleDataScope.getDataScopePerm();
        return userMapper.selectUserList(body.getPage(), RoleDataScope.getDataScopePerm(), body);
    }

    @Override
    public Page<SysUserVo> selectAllocatedList(SysUserBody body) {
        return userMapper.selectAllocatedList(body.getPage(), RoleDataScope.getDataScopePerm(), body);
    }

    @Override
    public Page<SysUserVo> selectUnallocatedList(SysUserBody body) {
        return userMapper.selectUnallocatedList(body.getPage(), RoleDataScope.getDataScopePerm(), body);
    }

    @Override
    public SysUserVo selectUserById(Long userId) {
        return userMapper.selectUserById(userId);
    }

    @Override
    public String selectUserRoleGroup(String userName) {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list)) {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    @Override
    public void checkUserDataScope(Long userId) {
        if (!RoleDataScope.isAdmin(SecurityUtils.getUserId())) {
            if (baseMapper.checkUserDataScope(userId, RoleDataScope.getDataScopePerm()) <= 0) {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    @Override
    @Transactional
    public boolean insertUser(SysUser user) {
        // 新增用户信息
        boolean result = save(user);
        // 新增用户与角色管理
        this.insertUserRole(user.getUserId(), user.getRoleIds());
        return result;
    }

    @Override
    @Transactional
    public int updateUser(SysUser user) {
        Long userId = user.getUserId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        this.insertUserRole(user.getUserId(), user.getRoleIds());

        //用户名不能修改
        user.setUserName(null);
        return userMapper.updateById(user);
    }

    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds) {
        for (Long userId : userIds) {
            if (RoleDataScope.isAdmin(userId)) {
                throw new ServiceException("不允许操作超级管理员用户");
            }
            checkUserDataScope(userId);
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        return userMapper.deleteUserByIds(userIds);
    }

    @Override
    public String checkUserNameUnique(Long userId, String userName) {
        if (userMapper.checkUserNameUnique(userId, userName) > 0) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public String checkPhoneUnique(Long userId, String phone) {
        if (userMapper.checkPhoneUnique(userId, phone) > 0) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public String checkEmailUnique(Long userId, String email) {
        if (userMapper.checkEmailUnique(userId, email) > 0) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    @Override
    public UserActiveVo getUserActive() {
        return userMapper.getUserActive(RoleDataScope.getDataScopePerm());
    }


    /**
     * 新增用户角色信息
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    private void insertUserRole(Long userId, Long[] roleIds) {
        if (StringUtils.isNotEmpty(roleIds)) {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<>(roleIds.length);
            for (Long roleId : roleIds) {
                list.add(new SysUserRole(userId, roleId));
            }
            userRoleMapper.batchUserRole(list);
        }
    }
}
